#!/usr/bin/python
#
# Apache Configuration Override (ApCO)
#
# This is meant to be called from /etc/rc.d/init.d/httpd
#
# Ran on Apache startup, this program overrides httpd.conf if it has been
# changed by a third party application.
#
# Events are logged and errors also reported by email. Log is recycled
# if over 100kB, previous log is kept.
#
#
# <!> Warning <!>
#
# When making changes to the Apache config., always change the source file
# in "app_dir" below. When you restart apache with "service httpd restart",
# this file will be copied to the apache directory ("apache_conf_dir" below).
#


#
#
# JM Rosengard - http://w2.syronex.com/jmr/technical/custom-apache-conf-vps
# (C)2006 Syronex


#
# Configuration
#

#
# apco installation directory
#
app_dir = '/usr/local/apco'

#
# Apache configuration directory
#
apache_conf_dir = '/etc/httpd/conf'

#
# Log files directory
#
log_dir = '/usr/local/apco/log'

#
# Apache configuration file name
#
conf_fn = 'httpd.conf'

#
# Email address for errors report
#
admin_email = 'alice@example.com'


import md5
import shutil 
import os.path
import mailer
import logging
import logging.handlers


class ApCO:
    def __init__(self):        
        self._src_fn = os.path.join(app_dir, conf_fn)
        self._tgt_fn = os.path.join(apache_conf_dir, conf_fn)
        self.init_log()
        self._log.info('Apache start requested.')
        self.override()

    def override(self):
        """
        Overrides the target file if it is not the same as the reference
        source file.
        """
        try:
            if self.target_changed():
                self.replace_target()
                self._log.warn(
                    'Target "%s" different from from source file: overriding.' % self._tgt_fn)
            else:
                self._log.info('No differences.')
                
        except Exception, cause:
            self._log.error(cause)
            alert = mailer.Message(
                ('Web Server Admin', admin_email),
                ('Web Server Admin', admin_email),
                'Apache Config. Override Error.', cause)
            alert.send()

    def target_changed(self):
        """
        The target file has been altered if its hash is different from
        the source file.
        """
        sf = open(self._src_fn)
        hasher = md5.new()
        hasher.update(''.join(sf.readlines()))
        s_md = hasher.digest()
        sf.close()
        tf = open(self._tgt_fn)
        hasher = md5.new()
        hasher.update(''.join(tf.readlines()))
        t_md = hasher.digest()
        tf.close()
        return s_md != t_md

    def replace_target(self):    
        shutil.copyfile(self._src_fn, self._tgt_fn)

    def init_log(self):
        self._log = logging.getLogger('com.syronex.apco')
        self._log.setLevel(logging.DEBUG)
        hdl = logging.handlers.RotatingFileHandler(    
            filename = os.path.join(log_dir, 'apco-events.log'),
            maxBytes = '100',
            backupCount = '2')
        hdl.setLevel(logging.DEBUG)
        hdl.setFormatter(
            logging.Formatter('%(asctime)s %(levelname)s %(message)s'))
        self._log.addHandler(hdl)



if __name__ == '__main__':
    override = ApCO()
